/*-----------------------------------------------------------------------------
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//     
//     (c) Copyright 2003 Xilinx, Inc.
//     All rights reserved.
//
//---------------------------------------------------------------------------*/
#include "xparameters.h"
#include "xio.h"
#include "xbasic_types.h"
#include <promread.h>

/*****************************************************************************
* Function name    :  read_helper (void) 
* returns          :  Void
* Created by       :  SUS
* Date Created     :  04/07/2004
* Description      :  Reads Platform FLASH  device serially 32 time and puts it 
*					: into 32-bit word. Then it looks for a USERSYNC word.
*				   	: Once a USERSYNC word is found it expects to find a address
*					: location.  The data is continually read and copied into the 
*					: memory start at memory location stated after the USERSYNC word.  
*					: this continues until another USERSYNC word or end of data is 
*					: detected.  If a another USERSYNC word is found then the process
*					: of copying data is repeated.  A end of data is detected by 
*					: finding a match to the END_USER_DATA from the 32-bit data.
*                  :  
*****************************************************************************/
Xuint32 prom_read_helper( Xuint8 num)
{
Xuint8 i;
Xuint32 data32;

for( i=0; i<num;i++)
	{
	//single bit is shifted into the 32-bit word. 
	data32 = (data32 << 1) + (XIo_In32(XPAR_PROMREAD_BASEADDR+0x08));
	//clock the PROM to output data
	XIo_Out32(XPAR_PROMREAD_BASEADDR, OE_HIGH | CCLK_HIGH | CE_LOW);
	XIo_Out32(XPAR_PROMREAD_BASEADDR, OE_HIGH | CCLK_LOW | CE_LOW);
	}

return data32;
	
}



/*****************************************************************************
* Function name    :  promread (Xuint read) 
* returns          :  Xuint32
* Created by       :  SUS
* Date Created     :  04/07/2004
* Description      :  Reads Platform FLASH  device serially 32 time and puts it 
*					: into 32-bit word. Then it looks for a USERSYNC word.
*				   	: Once a USERSYNC word is found it expects to find a address
*					: location.  The data is continually read and copied into the 
*					: memory start at memory location stated after the USERSYNC word.  
*					: this continues until another USERSYNC word or end of data is 
*					: detected.  If a another USERSYNC word is found then the process
*					: of copying data is repeated.  A end of data is detected by 
*					: finding a match to the END_USER_DATA from the 32-bit data.
*                  :  
*****************************************************************************/
Xuint32 promread (Xuint8 read) {

Xuint32 	i			= 0; // for loop control variable
Xuint32 	j; 
//Xuint32 addr		= 0; // used to increment the address offset after each 32-bit word is written to memory
Xuint32 addr_start	= 0; // used to store the starting address of the memory in the memory map
Xuint32 num_bytes 	= 0; // stores number of bytes in an address sections
Xuint8 addr_done 	= 0; // indicates the end of all the addres sections
Xuint8 overflow 	= 0; // indicates that a 32-bit word overflow has occured in the address sections

Xuint32 data32,data8 	= 0; 
Xuint32 data 	= 0; 
Xuint32 find,done, addr_sect_overflow 	= 0; 

// bit/byte  swaps the USER sync word for SYNC word detection

if(read==DATAREAD)
find = (DATASYNC);
else 
find = (ADDRSYNC);



	//sets the INIT and CCLK bits high
	XIo_Out32(XPAR_PROMREAD_BASEADDR, OE_HIGH | CCLK_LOW | CE_LOW);

	while(data32 != find)
	 	{
		data32=prom_read_helper(32);
		}

	if(read==DATAREAD)
		{
		data32=prom_read_helper(32);
		}
else
while(!done)
{
	//if(read==ADDRREAD)
	//	{
			data32=prom_read_helper(32);
			addr_start = data32;
			data32=prom_read_helper(32);
			num_bytes=data32;
			INFO("\n\r Address = 0x%x  num_bytes %x ",addr_start, num_bytes);
			while(addr_done !=1)
			{
			for(i=0; i<num_bytes/4;i++)
				{
				data32=prom_read_helper(32);
				XIo_Out32((4*i)+addr_start, data32);
				INFO("\n\r Address = 0x%x  Data %x",addr_start+(4*i), data32);
				}

			overflow = num_bytes%4;
			addr_sect_overflow = addr_sect_overflow + overflow;
			if(overflow>0) 
				{
				for(i=0; i<overflow;i++)
					{
					data8=prom_read_helper(8);
					data32 = data32 & (data8 << (32-(overflow*8)));
					}
					XIo_Out32(num_bytes+addr_start-overflow, data32);
					INFO("%d Overlow Address = 0x%x  Data %x\n\r ", 	overflow, num_bytes+addr_start-overflow, data32);
				overflow=0;
				}
			data32=prom_read_helper(32);
			addr_start=(data32);
			data32=prom_read_helper(32);
			num_bytes=(data32);
			if (addr_start ==0x0 && num_bytes==0x0) addr_done=1; else addr_done=0;
			INFO("\n\rEnd Addr section Address = 0x%x  Data %x addr_done %x\n\r ",addr_start, num_bytes, addr_done);
			}
			if (addr_sect_overflow%4 > 0)
				{
				for(i=0; i<(addr_sect_overflow%4);i++)
					{
					data32=prom_read_helper(8);
					}
				addr_sect_overflow=0;
				}

			while(data != END_PROM && data != find)
				{
				data=prom_read_helper(32);
				}
			if(data == END_PROM) done = 1;
				else done=0;

}
	 	//sets the INIT and CCLK bits LOW
		XIo_Out32(XPAR_PROMREAD_BASEADDR, OE_LOW | CCLK_LOW | CE_HIGH);
		return data32;

}